#ifndef ross_long_int_header
#define ross_long_int_header

/*Authored 2002    by  Kyle  Ross,  Undergraduate  Philosophy   Thesis
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/

#include <deque>
#include <iostream>
#include <string>

class long_int
{
public:
	long_int()
	:zero(true), value(1, 0)
	{}
	long_int(const long_int &rhs)
	:zero(rhs.zero), value(rhs.value)
	{}
	long_int(unsigned long int new_value)
	{
		if(new_value==0)
		{
			zero=true;
			value=std::deque<unsigned short int>(1, 0);
		}
		else
		{
			zero=false;
			while(new_value!=0)
			{
				value.push_front(new_value%10);
				new_value/=10;
			}
		}
	}


	const long_int& operator+=(const long_int& rhs)
	{
		if(zero && !rhs.zero)
		{
			value=rhs.value;
			zero=false;
		}
		else if(!rhs.zero)
		{
			std::deque<unsigned short int>::const_reverse_iterator i=value.rbegin(), j=rhs.value.rbegin();

			std::deque<unsigned short int> temp;

			unsigned short int carry=0, current;
			while(carry || i!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(value.rend()) || j!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(rhs.value.rend()))
			{
				current=carry;
				if(i!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(value.rend()))
				{
					current+=(*i);
					++i;
				}
				if(j!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(rhs.value.rend()))
				{
					current+=(*j);
					++j;
				}

				carry=current/10;
				current%=10;

				temp.push_front(current);
			}

			value=temp;
		}

		return *this;
	}


	const long_int operator+(const long_int& rhs) const
	{
		long_int temp=*this;
		temp+=rhs;
		return temp;
	}
	const long_int multiply_by_digit(unsigned short int digit) const
	{
		if(digit==0)
		{
			return long_int(0);
		}

		if(digit==1 || zero)
		{
			return *this;
		}

		if(digit>9)
		{
			std::cerr<<"cannot multiply by digit >9"<<std::endl;
			return *this;
		}

		std::deque<unsigned short int>::const_reverse_iterator i=value.rbegin();

		long_int temp;
		temp.value.clear();

		unsigned short int carry=0, current;

		while(carry || i!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(value.rend()))
		{
			current=carry;

			if(i!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(value.rend()))
			{
				current+=(*i)*digit;
				++i;
			}

			carry=current/10;
			current%=10;

			temp.value.push_front(current);
		}

		temp.zero=false;
		return temp;
	}

	const long_int shift_by_power_of_10(unsigned int digit) const
	{
		long_int temp=*this;

		if(!zero && !(digit==0))
		{
			while(digit)
			{
				temp.value.push_back(0);
				--digit;
			}
		}

		return temp;
	}
	const long_int& operator*=(const long_int& rhs)
	{
		if(zero)
		{
			return *this;
		}
		if(rhs.zero)
		{
			return *this=rhs;
		}

		long_int mult_digits[10];
		for(unsigned short int i=0; i<=9; ++i)
		{
			mult_digits[i]=multiply_by_digit(i);
		}

		std::deque<unsigned short int>::const_reverse_iterator i=rhs.value.rbegin();

		unsigned int power_of_10=0;

		long_int temp(0);

		while(i!=static_cast<std::deque<unsigned short int>::const_reverse_iterator>(rhs.value.rend()))
		{
			temp+=mult_digits[*i].shift_by_power_of_10(power_of_10);
			++power_of_10;
			++i;
		}

		return *this=temp;
	}
	const long_int operator*(const long_int& rhs) const
	{
		long_int temp=*this;
		temp*=rhs;
		return temp;
	}

	const long_int& operator++()
	{
		unsigned short int carry=1, current;

		std::deque<unsigned short int>::reverse_iterator i=value.rbegin();

		while(carry && i!=static_cast<std::deque<unsigned short int>::reverse_iterator>(value.rend()))
		{
			current=carry+(*i);
			carry=current/10;
			current%=10;
			(*i)=current;
			++i;
		}

		if(carry)
		{
			value.push_front(1);
		}
		
		return *this;
	}
	bool operator==(const long_int& rhs) const
	{
		return value==rhs.value;
	}
	bool operator<(const long_int& rhs) const
	{
		if(zero)
		{
			if(rhs.zero)
			{
				return false;
			}
			else
			{
				return true;
			}
		}
		else if(rhs.zero)
		{
			return false;
		}

		if(value.size()<rhs.value.size())
		{
			return true;
		}
		else if(value.size()>rhs.value.size())
		{
			return false;
		}
		else if(value==rhs.value)
		{
			return false;
		}

		std::deque<unsigned short int>::const_iterator i=value.begin(), j=rhs.value.begin();
		while(true)
		{
			if((*i)<(*j))
			{
				return true;
			}
			else if((*i)>(*j))
			{
				return false;
			}
			else
			{
				++i;
				++j;
			}
		}
	}
	void print(std::ostream& out) const
	{
		if(zero)
		{
			out<<'0';
		}
		else
		{
			for(std::deque<unsigned short int>::const_iterator i=value.begin(); i!=value.end(); ++i)
			{
				out<<(*i);
			}
		}
	}
	//begin Owen Kellett modifications
	std::string toString() const
	{
		std::string result;

		if(zero)
		{
			result = "0";
		}
		else
		{
			for(std::deque<unsigned short int>::const_iterator i=value.begin(); i!=value.end(); ++i)
			{
				result+=(char)((*i)+48);
			}
		}
		return result;
	}
	//end Owen Kellett modifications

private:
	bool zero;
	std::deque<unsigned short int> value;

};
std::ostream& operator<<(std::ostream &out, const long_int &to_print)
{
	to_print.print(out);
	return out;
}
const long_int pow(const long_int& base, const long_int& exponent)
{
	if(exponent==0)
	{
		return 1;
	}

	long_int counter=0, ret_val=1;
	while(!(counter==exponent))
	{
		ret_val*=base;
		++counter;
	}

	return ret_val;
}
const long_int permutations(unsigned int n, unsigned int r)
{
	if(r<=0)
	{
		return long_int(1);
	}
	if(r>n)
	{
		return long_int(0);
	}

	long_int result=1;
	long_int multiplier=n-r+1;
	for(int i=n-r+1; i<=n; ++i)
	{
		result*=multiplier;
		++multiplier;
	}
	return result;
}


#endif
