#ifndef kellett_machine_storage_header
#define kellett_machine_storage_header

/*Authored 2003    by Owen Kellett, Undergraduate Research
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/
//implements a simple class for tracking statistics of turing machines during runs

#include <fstream>
#include <iostream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <gmpxx.h>

#include "turing_machine.hpp"
#include "calculations.hpp"

/*template <typename T>
class machineStorage
{
public:
	//the reasons for elimination of machines
	enum reason{candidate=0, not_in_standard_position, holdout, simple_nonproductive, complex_nonproductive, empty_tape, first_write_1, first_move_right, backTracker, simpleLoop, subsetLoop, christmasTree, alternateChristmasTree, leaningChristmasTree, tripleSweepChristmasTree, quadrupleSweepChristmasTree, quintupleSweepChristmasTree, counter};
	enum machine_type{implicit_halt, explicit_halt};
	enum bestType{ones=0, transitions};
	//names of reasons for display ... this must be the same length as the enumerated reasons
	static const std::string names[];
	static const std::string bestNames[];
	private:
		//this must match the length of the enumerated reasons
		static const int num_reasons=18;
	public:

	//initialise all counts to 0
	machineStorage(std::string d, bool o)
	:counts(num_reasons, 0), directory(d), onlyStoreHoldouts(o), championCounts(2,0)
	{}

	//print out the stats
	void print(const turing_machine<T> &machine, reason why)
	{
		if(!onlyStoreHoldouts || why == holdout)
		{
			std::string fileName = directory + names[why] + counts[why].toString() + ".tmo";
			std::ofstream file(fileName.c_str());
			machine.printTMO(file);
			file.close();
			counts[why]+=1;
		}
	}
	void printChampion(const turing_machine<T> &machine, bestType why, long_int value)
	{
		std::string fileName = directory + bestNames[why] + ".tmo";
		std::ofstream file(fileName.c_str());
		machine.printTMO(file);
		file.close();

		std::string fileName2 = directory + bestNames[why] + "Progression/" + value.toString() + "-championFound" + championCounts[why].toString() + ".tmo";
		std::ofstream file2(fileName2.c_str());
		machine.printTMO(file2);
		file2.close();
		championCounts[why]+=1;
	}
private:
	std::string directory;
	std::vector<long_int> counts;
	std::vector<long_int> championCounts;
	bool onlyStoreHoldouts;
};

template <typename T>
const std::string machineStorage<T>::names[]={"halterCandidate/", "nonStandardHalt/", "holdout/", "ssTransition/", "ssPrimeTransition/", "emptyTape/", "firstWrite1/", "firstMoveR/", "backTracker/", "simpleLoop/", "subsetLoop/", "christmasTree/", "alternateChristmasTree/", "leaningChristmasTree/", "tripleSweepChristmasTree/", "quadrupleSweepChristmasTree/", "quintupleSweepChristmasTree/", "counter/"};
template <typename T>
const std::string machineStorage<T>::bestNames[]={"champion", "transitionsChampion"};*/

template <typename T>
class championLog
{
public:
	enum bestType{ones=0, transitions};
	static const std::string bestNames[];

	championLog(std::string d)
	:directory(d), counts(2,0)
	{
		for(int j = 0; j < 2; j++)
		{
			std::string dirName1 = directory + bestNames[j] + "Progression";
			if(mkdir(dirName1.c_str(), 0744) == -1)
				std::cerr << "problem creating progression directory" << std::endl;
		}
	}

	void printChampion(const turing_machine<T> &machine, bestType why, mpz_class value)
	{
		std::string fileName = directory + bestNames[why] + ".tmo";
		std::ofstream file(fileName.c_str());
		machine.printTMO(file);
		file.close();

		std::string fileName2 = directory + bestNames[why] + "Progression/" + mpzToString(value) + "-championFound" + mpzToString(counts[why]) + ".tmo";
		std::ofstream file2(fileName2.c_str());
		machine.printTMO(file2);
		file2.close();
		counts[why]+=1;
	}

private:
	std::string directory;
	std::vector<mpz_class> counts;
};

template <typename T>
const std::string championLog<T>::bestNames[]={"champion", "transitionsChampion"};

template <typename T>
class outputLog
{
public:
	outputLog(std::string d)
	:directory(d), counts(0)
	{}

	void print(const turing_machine<T> &machine)
	{
		std::string fileName = directory + mpzToString(counts) + ".tmo";
		std::ofstream file(fileName.c_str());
		machine.printTMO(file);
		file.close();
		counts+=1;
	}

private:
	std::string directory;
	mpz_class counts;
};


#endif
