#ifndef ross_stats_header
#define ross_stats_header

/*Authored 2002    by  Kyle  Ross,  Undergraduate  Philosophy   Thesis
(Advisor: Professor Bram van Heuveln)

Modified 2003    by Owen Kellett, Undergraduate Research
(Advisor: Professor Bram van Heuveln)


This  file is part of the   "Generic Turing Machine Implementation for
Use in the Exploration of the Busy Beaver Problem".

The "Generic Turing Machine Implementation for  Use in the Exploration
of the Busy Beaver Problem" is free  software; you can redistribute it
and/or modify it under the terms of the  GNU General Public License as
published   by the Free Software Foundation;   either version 2 of the
License, or (at your option) any later version.

This programme is distributed in the hope that it  will be useful, but
WITHOUT    ANY  WARRANTY;  without   even  the    implied warranty  of
MERCHANTABILITY or FITNESS FOR  A  PARTICULAR  PURPOSE.  See  the  GNU
General Public License for more details.

You should have received   a copy of  the  GNU General Public  License
along  with  this   programme; if  not,  write   to the Free  Software
Foundation, Inc., 59  Temple Place, Suite  330, Boston, MA  02111-1307
USA (or visit http://www.fsf.org/licenses/licenses.html)

Created as part of the Busy Beaver Project:
Bram van Heuveln <heuveb@rpi.edu>
Boleshaw Szymanski <szymansk@cs.rpi.edu>
Selmer Bringsjord <selmer@rpi.edu>
Carlos Varela <cvarela@cs.rpi.edu>
Owen Kellett <kelleo@rpi.edu>
Shailesh Kelkar <kelkas@rpi.edu>
Kyle Ross <rossk2@rpi.edu>
Rensselaer Reasoning Group
Department of Cognitive Science
Department of Computer Science
Rensselaer Polytechnic Institute
Troy, New York 12180
U.S.A.
*/
//implements a simple class for tracking statistics of turing machines during runs

#include <cmath>
#include <iostream>
#include <vector>
#include <gmpxx.h>

template <typename T>
class turing_machine;
class tm_stats
{
public:
	//the reasons for elimination of machines
	enum reason{candidate=0, not_in_standard_position, holdout, simple_nonproductive, complex_nonproductive, empty_tape, first_write_1, first_move_right, backTracker, subsetLoop, simpleLoop, christmasTree, alternateChristmasTree, leaningChristmasTree, tripleSweepChristmasTree, quadrupleSweepChristmasTree, quintupleSweepChristmasTree, sextupleSweepChristmasTree, septupleSweepChristmasTree, octupleSweepChristmasTree, counter, nestedChristmasTree};
	enum machine_type{implicit_halt, explicit_halt};
	//names of reasons for display ... this must be the same length as the enumerated reasons
	static const char* names[];
	private:
		//this must match the length of the enumerated reasons
		static const int num_reasons=22;
	public:

	//initialise all counts to 0
	tm_stats()
	:counts(num_reasons, 0)
	{}

	//increment a count by the specified amount
	void increment(reason why, mpz_class amount)
	{
		//std::cout<<"eliminated for reason: "<<names[why]<<" representing: "<<amount<<std::endl<<std::endl;
		counts[why]+=amount;
	}
	void reset()
	{
		for(int i=0; i<num_reasons; ++i)
		{
			counts[i]=0;
		}
	}
	//print out the stats
	void print(std::ostream &out) const
	{
		mpz_class accum=0;

		out<<"elimination statistics"<<std::endl;
		for(int i=0; i<num_reasons; ++i)
		{
			out<<' '<<names[i]<<": "<<counts[i]<<std::endl;
			accum+=counts[i];
		}
		out<<" total: "<<accum<<std::endl;
	}
private:
	std::vector<mpz_class> counts;
};

const char* tm_stats::names[]={"halter/candidate", "non-standard halt", "holdout", "ss transition", "ss-prime transition", "empty-tape", "first-write-1", "first-move-R", "backTracker", "subsetLoop", "simpleLoop", "christmasTree", "alternateChristmasTree", "leaningChristmasTree", "tripleSweepChristmasTree", "quadrupleSweepChristmasTree", "quintupleSweepChristmasTree", "sextupleSweepChristmasTree", "septupleSweepChristmasTree", "octupleSweepChristmasTree", "counter", "nestedChristmasTree"};


std::ostream& operator<<(std::ostream &out, const tm_stats &to_print)
{
	to_print.print(out);
	return out;
}

#endif
