#include <fstream>
#include <iostream>
#include <iterator>
#include <list>
#include <vector>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>

#include "../bb/stats.hpp"
tm_stats g_counter, c_counter;

#include "../bb/infinite_tape.hpp"
#include "../bb/state.hpp"
#include "../bb/turing_machine.hpp"
#include "../bb/non_halt_detection.hpp"
#include "../bb/non_halt_data_structures.hpp"
#include "../bb/machine_storage.hpp"
#include "../bb/watch.hpp"

turing_machine<unsigned short> parseMachine(std::string filename);
void testHoldouts(std::string holdoutDir, int numHoldouts);
void runTests(turing_machine<unsigned short> &current_machine,
		mpz_class machineNum,
		tm_stats &g_counter,
		tm_stats &c_counter,
		std::vector< std::vector<mpz_class> > &newNonHalters);
void displayNonHaltStats(tm_stats &g_counter, tm_stats &c_counter, std::vector< std::vector<mpz_class> > &newNonHalters);

int main()
{
	testHoldouts("/home/kelleo/UpdateFiles/URP/B5byhand/holdouts/", 98);

	return 0;
}

turing_machine<unsigned short> parseMachine(std::string filename)
{
	std::ifstream file(filename.c_str());
	std::string temp;
	int states;

	file >> temp;
	while(temp != "/*NumStates")
		file >> temp;
	file >> temp;
	file >> states;

	turing_machine<unsigned short> machine(states+1,0);
	machine.set_halt_state(states);

	while(temp != "start")
		file >> temp;
	file >> temp;
	file >> temp;

	while(1)
	{
		int originalState, nextState, readChar, writeChar;
		char read, action;
		std::string firstTemp;
		turing_machine<unsigned short>::move_type move;
		bool writeAction;

		file >> firstTemp;
		if(firstTemp == "/*EndGraphOutput*/")
			break;
		else
			originalState = firstTemp[0] - 48;
		file >> temp;
		file >> nextState;
		file >> temp;

		if(file.eof())
			break;
		read = temp[8];
		action = temp[10];
		switch(read)
		{
			case '0': readChar = 0; break;
			case '1': readChar = 1; break;
		}
		switch(action)
		{
			case '0': writeChar = 0; writeAction = true; break;
			case '1': writeChar = 1; writeAction = true; break;
			case 'L': move = turing_machine<unsigned short>::left; writeAction = false; break;
			case 'R': move = turing_machine<unsigned short>::right; writeAction = false; break;
		}
		if(writeAction)
			machine.add_write_transition(originalState,readChar,writeChar,nextState);
		else
			machine.add_move_transition(originalState,readChar,move,nextState);
	}

	return machine;
}

void testHoldouts(std::string holdoutDir, int numHoldouts)
{
	std::string filename;
	mpz_class i = 0;
	mpz_class max = numHoldouts;
	int status = 0;

	tm_stats g_counter, c_counter;
	std::vector< std::vector<mpz_class> > newNonHalters(14);
	for(i; i < max; i+=1, status++)
	{
		filename = holdoutDir + mpzToString(i) + ".tmo";
		turing_machine<unsigned short> current_machine(parseMachine(filename));
		runTests(current_machine, i, g_counter, c_counter, newNonHalters);
		if(status%1 == 0)
			std::cout << "machine " << status << " done" << std::endl;
	}

	displayNonHaltStats(g_counter, c_counter, newNonHalters);
}

void runTests(turing_machine<unsigned short> &current_machine,
		mpz_class machineNum,
		tm_stats &g_counter,
		tm_stats &c_counter,
		std::vector< std::vector<mpz_class> > &newNonHalters)
{
	if(backTrack(current_machine, 40))
	{
		newNonHalters[0].push_back(machineNum);
		g_counter.increment(tm_stats::backTracker, current_machine.number_represented());
		c_counter.increment(tm_stats::backTracker, 1);
	}
	/*else if(loopStates(current_machine))
	{
		newNonHalters[1].push_back(machineNum);
		g_counter.increment(tm_stats::subsetLoop, current_machine.number_represented());
		c_counter.increment(tm_stats::subsetLoop, 1);
	}
	else if(simpleLoopCheck(current_machine, 5000))
	{
		newNonHalters[2].push_back(machineNum);
		g_counter.increment(tm_stats::simpleLoop, current_machine.number_represented());
		c_counter.increment(tm_stats::simpleLoop, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 1, 200000))
	{
		newNonHalters[3].push_back(machineNum);
		g_counter.increment(tm_stats::christmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::christmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 2, 200000))
	{
		newNonHalters[4].push_back(machineNum);
		g_counter.increment(tm_stats::alternateChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::alternateChristmasTree, 1);
	}
	else if(leaningChristmasTree(current_machine, 200000))
	{
		newNonHalters[5].push_back(machineNum);
		g_counter.increment(tm_stats::leaningChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::leaningChristmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 3, 200000))
	{
		newNonHalters[6].push_back(machineNum);
		g_counter.increment(tm_stats::tripleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::tripleSweepChristmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 4, 200000))
	{
		newNonHalters[7].push_back(machineNum);
		g_counter.increment(tm_stats::quadrupleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::quadrupleSweepChristmasTree, 1);
	}*/
	else if(multiSweepChristmasTree(current_machine, 5, 200000))
	{
		newNonHalters[8].push_back(machineNum);
		g_counter.increment(tm_stats::quintupleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::quintupleSweepChristmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 6, 200000))
	{
		newNonHalters[9].push_back(machineNum);
		g_counter.increment(tm_stats::sextupleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::sextupleSweepChristmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 7, 200000))
	{
		newNonHalters[10].push_back(machineNum);
		g_counter.increment(tm_stats::septupleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::septupleSweepChristmasTree, 1);
	}
	else if(multiSweepChristmasTree(current_machine, 8, 200000))
	{
		newNonHalters[11].push_back(machineNum);
		g_counter.increment(tm_stats::octupleSweepChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::octupleSweepChristmasTree, 1);
	}
	else if(counter(current_machine, 100000))
	{
		newNonHalters[12].push_back(machineNum);
		g_counter.increment(tm_stats::counter, current_machine.number_represented());
		c_counter.increment(tm_stats::counter, 1);
	}
	else if(nestedChristmasTree(current_machine, 50000))
	{
		newNonHalters[13].push_back(machineNum);
		g_counter.increment(tm_stats::nestedChristmasTree, current_machine.number_represented());
		c_counter.increment(tm_stats::nestedChristmasTree, 1);
	}
	/*else
	{
		current_machine.resetMachine();
		int step_counter = 0;
		while(step_counter<10000000 && current_machine.run_one()!=turing_machine<d_type>::halt)
		{
			++step_counter;
		}
		if(current_machine.get_status()==turing_machine<d_type>::halt)
		{
			std::cout << "A new halter! Machine " << machineNum << " prints out ";
			std::cout << current_machine.count_occurrences(1) << " 1's, and halts after ";
			std::cout << current_machine.get_steps_count() << "steps" << std::endl;
		}
	}*/
}

void displayNonHaltStats(tm_stats &g_counter, tm_stats &c_counter, std::vector< std::vector<mpz_class> > &newNonHalters)
{
	std::cout << "observed " << c_counter << std::endl;
	std::cout << "representative " << g_counter << std::endl;
	for(int i = 0; i < newNonHalters.size(); i++)
	{
		switch(i)
		{
			case 0: std::cout << "Backtrackers : "; break;
			case 1: std::cout << "SubsetLoops : "; break;
			case 2: std::cout << "SimpleLoops : "; break;
			case 3: std::cout << "ChristmasTrees : "; break;
			case 4: std::cout << "AlternateChristmasTrees : "; break;
			case 5: std::cout << "LeaningChristmasTrees : "; break;
			case 6: std::cout << "TripleSweepChristmasTrees : "; break;
			case 7: std::cout << "QuadrupleSweepChristmasTrees : "; break;
			case 8: std::cout << "QuintupleSweepChristmasTrees : "; break;
			case 9: std::cout << "SextupleSweepChristmasTrees : "; break;
			case 10: std::cout << "SeptupleSweepChristmasTrees : "; break;
			case 11: std::cout << "OctupleSweepChristmasTrees : "; break;
			case 12: std::cout << "Counters : "; break;
			case 13: std::cout << "NestedChristmasTrees : "; break;
		}
		for(int j = 0; j < newNonHalters[i].size(); j++)
		{
			std::cout << newNonHalters[i][j] << ' ';
		}
		std::cout << std::endl;
	}
}
